@@ -31,23 +31,24 @@ static char const * const width_str[] = {
 static char const all_width_types[] = "hkboqs";
 
 
-int lsx_biquad_getopts(sox_effect_t * effp, int n, char **argv,
+int lsx_biquad_getopts(sox_effect_t * effp, int argc, char **argv,
     int min_args, int max_args, int fc_pos, int width_pos, int gain_pos,
     char const * allowed_width_types, filter_t filter_type)
 {
   priv_t * p = (priv_t *)effp->priv;
   char width_type = *allowed_width_types;
   char dummy, * dummy_p;     /* To check for extraneous chars. */
+  --argc, ++argv;
 
   p->filter_type = filter_type;
-  if (n < min_args || n > max_args ||
-      (n > fc_pos    && ((p->fc = lsx_parse_frequency(argv[fc_pos], &dummy_p)) <= 0 || *dummy_p)) ||
-      (n > width_pos && ((unsigned)(sscanf(argv[width_pos], "%lf%c %c", &p->width, &width_type, &dummy)-1) > 1 || p->width <= 0)) ||
-      (n > gain_pos  && sscanf(argv[gain_pos], "%lf %c", &p->gain, &dummy) != 1) ||
+  if (argc < min_args || argc > max_args ||
+      (argc > fc_pos    && ((p->fc = lsx_parse_frequency(argv[fc_pos], &dummy_p)) <= 0 || *dummy_p)) ||
+      (argc > width_pos && ((unsigned)(sscanf(argv[width_pos], "%lf%c %c", &p->width, &width_type, &dummy)-1) > 1 || p->width <= 0)) ||
+      (argc > gain_pos  && sscanf(argv[gain_pos], "%lf %c", &p->gain, &dummy) != 1) ||
       !strchr(allowed_width_types, width_type) || (width_type == 's' && p->width > 1))
     return lsx_usage(effp);
   p->width_type = strchr(all_width_types, width_type) - all_width_types;
-  if (p->width_type >= strlen(all_width_types))
+  if ((size_t)p->width_type >= strlen(all_width_types))
     p->width_type = 0;
   if (p->width_type == width_bw_kHz) {
     p->width *= 1000;
@@ -57,7 +58,7 @@ int lsx_biquad_getopts(sox_effect_t * effp, int n, char **argv,
 }
 
 
-int lsx_biquad_start(sox_effect_t * effp)
+static int start(sox_effect_t * effp)
 {
   priv_t * p = (priv_t *)effp->priv;
   /* Simplify: */
@@ -67,12 +68,23 @@ int lsx_biquad_start(sox_effect_t * effp)
   p->a2 /= p->a0;
   p->a1 /= p->a0;
 
+  p->o2 = p->o1 = p->i2 = p->i1 = 0;
+  return SOX_SUCCESS;
+}
+
+
+int lsx_biquad_start(sox_effect_t * effp)
+{
+  priv_t * p = (priv_t *)effp->priv;
+
+  start(effp);
+
   if (effp->global_info->plot == sox_plot_octave) {
     printf(
       "%% GNU Octave file (may also work with MATLAB(R) )\n"
       "Fs=%g;minF=10;maxF=Fs/2;\n"
       "sweepF=logspace(log10(minF),log10(maxF),200);\n"
-      "[h,w]=freqz([%g %g %g],[1 %g %g],sweepF,Fs);\n"
+      "[h,w]=freqz([%.15e %.15e %.15e],[1 %.15e %.15e],sweepF,Fs);\n"
       "semilogx(w,20*log10(h))\n"
       "title('SoX effect: %s gain=%g frequency=%g %s=%g (rate=%g)')\n"
       "xlabel('Frequency (Hz)')\n"
@@ -93,7 +105,7 @@ int lsx_biquad_start(sox_effect_t * effp)
       "set xlabel 'Frequency (Hz)'\n"
       "set ylabel 'Amplitude Response (dB)'\n"
       "Fs=%g\n"
-      "b0=%g; b1=%g; b2=%g; a1=%g; a2=%g\n"
+      "b0=%.15e; b1=%.15e; b2=%.15e; a1=%.15e; a2=%.15e\n"
       "o=2*pi/Fs\n"
       "H(f)=sqrt((b0*b0+b1*b1+b2*b2+2.*(b0*b1+b1*b2)*cos(f*o)+2.*(b0*b2)*cos(2.*f*o))/(1.+a1*a1+a2*a2+2.*(a1+a1*a2)*cos(f*o)+2.*a2*cos(2.*f*o)))\n"
       "set logscale x\n"
@@ -106,7 +118,6 @@ int lsx_biquad_start(sox_effect_t * effp)
       , p->b0, p->b1, p->b2, p->a1, p->a2);
     return SOX_EOF;
   }
-  p->o2 = p->o1 = p->i2 = p-> i1 = 0;
   return SOX_SUCCESS;
 }
 
@@ -124,3 +135,24 @@ int lsx_biquad_flow(sox_effect_t * effp, const sox_sample_t *ibuf,
   }
   return SOX_SUCCESS;
 }
+
+static int create(sox_effect_t * effp, int argc, char * * argv)
+{
+  priv_t             * p = (priv_t *)effp->priv;
+  double             * d = &p->b0;
+  char               c;
+
+  --argc, ++argv;
+  if (argc == 6)
+    for (; argc && sscanf(*argv, "%lf%c", d, &c) == 1; --argc, ++argv, ++d);
+  return argc? lsx_usage(effp) : SOX_SUCCESS;
+}
+
+sox_effect_handler_t const * lsx_biquad_effect_fn(void)
+{
+  static sox_effect_handler_t handler = {
+    "biquad", "b0 b1 b2 a0 a1 a2", 0,
+    create, start, lsx_biquad_flow, NULL, NULL, NULL, sizeof(priv_t)
+  };
+  return &handler;
+}
