@@ -25,19 +25,20 @@ typedef struct {
 /*
  * Process options: dcshift (double) type (amplitude, power, dB)
  */
-static int sox_dcshift_getopts(sox_effect_t * effp, int n, char **argv)
+static int sox_dcshift_getopts(sox_effect_t * effp, int argc, char **argv)
 {
     priv_t * dcs = (priv_t *) effp->priv;
     dcs->dcshift = 1.0; /* default is no change */
     dcs->uselimiter = 0; /* default is no limiter */
 
-    if (n < 1)
+  --argc, ++argv;
+    if (argc < 1)
       return lsx_usage(effp);
 
-    if (n && (!sscanf(argv[0], "%lf", &dcs->dcshift)))
+    if (argc && (!sscanf(argv[0], "%lf", &dcs->dcshift)))
       return lsx_usage(effp);
 
-    if (n>1)
+    if (argc>1)
     {
         if (!sscanf(argv[1], "%lf", &dcs->limitergain))
           return lsx_usage(effp);
@@ -123,17 +124,9 @@ static int sox_dcshift_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_s
                 *obuf++ = sample;
             }
     }
-    else
-    {
-        /* quite basic, with clipping */
-        for (;len>0; len--)
-        {
-                double f;
-
-                f = dcshift * SOX_SAMPLE_MAX + *ibuf++;
-                SOX_SAMPLE_CLIP_COUNT(f, dcs->clipped);
-                *obuf++ = f;
-        }
+    else for (; len > 0; --len) {            /* quite basic, with clipping */
+      double d = dcshift * (SOX_SAMPLE_MAX + 1.) + *ibuf++;
+      *obuf++ = SOX_ROUND_CLIP_COUNT(d, effp->clips);
     }
     return SOX_SUCCESS;
 }
@@ -170,9 +163,9 @@ static int sox_dcshift_stop(sox_effect_t * effp)
 static sox_effect_handler_t sox_dcshift_effect = {
    "dcshift",
    "shift [ limitergain ]\n"
-   "       The peak limiter has a gain much less than 1.0 (ie 0.05 or 0.02) which is only\n"
-   "       used on peaks to prevent clipping. (default is no limiter)",
-   SOX_EFF_MCHAN,
+   "\tThe peak limiter has a gain much less than 1.0 (ie 0.05 or 0.02) which\n"
+   "\tis only used on peaks to prevent clipping. (default is no limiter)",
+   SOX_EFF_MCHAN | SOX_EFF_GAIN,
    sox_dcshift_getopts,
    sox_dcshift_start,
    sox_dcshift_flow,
@@ -181,7 +174,7 @@ static sox_effect_handler_t sox_dcshift_effect = {
   NULL, sizeof(priv_t)
 };
 
-const sox_effect_handler_t *sox_dcshift_effect_fn(void)
+const sox_effect_handler_t *lsx_dcshift_effect_fn(void)
 {
     return &sox_dcshift_effect;
 }
