@@ -1,24 +1,22 @@
-/*
- * Common routines for G.721 and G.723 conversions.
+/* Common routines for G.721 and G.723 conversions.
  *
  * (c) SoX Contributors
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or (at
+ * your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this library; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, write to the Free Software
- * Foundation, Fifth Floor, 51 Franklin Street, Boston, MA 02111-1301,
- * USA.
  *
- * 
  * This code is based on code from Sun, which came with the following
  * copyright notice:
  * -----------------------------------------------------------------------
@@ -90,7 +88,7 @@ static inline int log2plus1(int val)
 
         return r + 1;
 }
-  
+
 /*
  * quan()
  *
@@ -99,7 +97,7 @@ static inline int log2plus1(int val)
  *
  * Using linear search for simple coding.
  */
-static int quan(int val, short *table, int size)
+static int quan(int val, short const *table, int size)
 {
         int             i;
 
@@ -224,7 +222,7 @@ int step_size(struct g72x_state *state_ptr)
  * size scale factor division operation is done in the log base 2 domain
  * as a subtraction.
  */
-int quantize(int d, int y, short *table, int size)
+int quantize(int d, int y, short const *table, int size)
 {
         short           dqm;    /* Magnitude of 'd' */
         short           exp;    /* Integer part of base 2 log of 'd' */
@@ -295,7 +293,7 @@ int reconstruct(int sign, int dqln, int y)
  *
  * updates the state variables for each output code
  */
-void update(int code_size, int y, int wi, int fi, int dq, int sr, 
+void update(int code_size, int y, int wi, int fi, int dq, int sr,
             int dqsez, struct g72x_state *state_ptr)
 {
         int             cnt;
@@ -498,7 +496,7 @@ void update(int code_size, int y, int wi, int fi, int dq, int sr,
  * Return:
  *      adjusted A-law or u-law compressed sample.
  */
-int tandem_adjust_alaw(int sr, int se, int y, int i, int sign, short *qtab)
+int tandem_adjust_alaw(int sr, int se, int y, int i, int sign, short const *qtab)
 {
         unsigned char   sp;     /* A-law compressed 8-bit code */
         short           dx;     /* prediction error */
@@ -540,7 +538,7 @@ int tandem_adjust_alaw(int sr, int se, int y, int i, int sign, short *qtab)
         }
 }
 
-int tandem_adjust_ulaw(int sr, int se, int y, int i, int sign, short *qtab)
+int tandem_adjust_ulaw(int sr, int se, int y, int i, int sign, short const *qtab)
 {
         unsigned char   sp;     /* u-law compressed 8-bit code */
         short           dx;     /* prediction error */
