@@ -26,14 +26,15 @@ typedef struct {
 /*
  * Process options
  */
-static int sox_trim_getopts(sox_effect_t * effp, int n, char **argv)
+static int sox_trim_getopts(sox_effect_t * effp, int argc, char **argv)
 {
     priv_t * trim = (priv_t *) effp->priv;
+  --argc, ++argv;
 
     /* Do not know sample rate yet so hold off on completely parsing
      * time related strings.
      */
-    switch (n) {
+    switch (argc) {
         case 2:
             trim->length_str = lsx_malloc(strlen(argv[1])+1);
             strcpy(trim->length_str,argv[1]);
@@ -82,6 +83,7 @@ static int sox_trim_start(sox_effect_t * effp)
     trim->index = 0;
     trim->trimmed = 0;
 
+    effp->out_signal.length = trim->length;
     return (SOX_SUCCESS);
 }
 
@@ -128,7 +130,7 @@ static int sox_trim_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_samp
     } /* !trimmed */
 
     if (trim->trimmed || start_trim) {
-        if (trim->length && ((trim->trimmed+done) >= trim->length)) {
+        if (trim->length_str && ((trim->trimmed+done) >= trim->length)) {
             /* Since we know the end is in this block, we set done
              * to the desired length less the amount already read.
              */
@@ -146,7 +148,7 @@ static int sox_trim_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_samp
     return result;
 }
 
-static int kill(sox_effect_t * effp)
+static int lsx_kill(sox_effect_t * effp)
 {
     priv_t * trim = (priv_t *) effp->priv;
 
@@ -168,12 +170,12 @@ void sox_trim_clear_start(sox_effect_t * effp)
     trim->start = 0;
 }
 
-const sox_effect_handler_t *sox_trim_effect_fn(void)
+const sox_effect_handler_t *lsx_trim_effect_fn(void)
 {
   static sox_effect_handler_t handler = {
-    "trim", "start [length]", SOX_EFF_MCHAN|SOX_EFF_LENGTH,
+    "trim", "start [length]", SOX_EFF_MCHAN | SOX_EFF_LENGTH | SOX_EFF_MODIFY,
     sox_trim_getopts, sox_trim_start, sox_trim_flow,
-    NULL, NULL, kill, sizeof(priv_t)
+    NULL, NULL, lsx_kill, sizeof(priv_t)
   };
   return &handler;
 }
