@@ -300,12 +300,18 @@ static int startwrite(sox_format_t * ft)
     }
     quality = ft->encoding.compression;
   }
-#include "vorbis1.h"
+#define IGNORE_WARNING \
+  if (vorbis_encode_init_vbr(&ve->vi, ft->signal.channels, ft->signal.rate + .5, quality / 10))
+#include "ignore-warning.h"
+  {
+    lsx_fail_errno(ft, SOX_EFMT, "libVorbis cannot encode this sample-rate or # of channels");
+    return SOX_EOF;
+  }
 
   vorbis_analysis_init(&ve->vd, &ve->vi);
   vorbis_block_init(&ve->vd, &ve->vb);
 
-  ogg_stream_init(&ve->os, rand());     /* Random serial number */
+  ogg_stream_init(&ve->os, INT_MAX & (int)RANQD1);  /* Random serial number */
 
   if (write_vorbis_header(ft, ve) == HEADER_ERROR) {
     lsx_fail_errno(ft, SOX_EHDR,
@@ -390,10 +396,10 @@ static int seek(sox_format_t * ft, uint64_t offset)
   return ov_pcm_seek(vb->vf, (ogg_int64_t)(offset / ft->signal.channels))? SOX_EOF:SOX_SUCCESS;
 }
 
-SOX_FORMAT_HANDLER(vorbis)
+LSX_FORMAT_HANDLER(vorbis)
 {
   static const char *names[] = {"vorbis", "ogg", NULL};
-  static unsigned encodings[] = {SOX_ENCODING_VORBIS, 0, 0};
+  static const unsigned encodings[] = {SOX_ENCODING_VORBIS, 0, 0};
   static sox_format_handler_t handler = {SOX_LIB_VERSION_CODE,
     "Xiph.org's ogg-vorbis lossy compression", names, 0,
     startread, read_samples, stopread,
